@@ -7,6 +7,8 @@
 """
 import datetime
 import itertools
+import logging
+import random
 import socket
 import time
 import warnings
@@ -14,6 +16,8 @@ import warnings
 from SoftLayer import exceptions
 from SoftLayer.managers import ordering
 from SoftLayer import utils
+
+LOGGER = logging.getLogger(__name__)
 # pylint: disable=no-self-use
 
 
@@ -192,7 +196,9 @@ class VSManager(utils.IdentifierMixin, object):
                 'primaryIpAddress,'
                 '''networkComponents[id, status, speed, maxSpeed, name,
                                      macAddress, primaryIpAddress, port,
-                                     primarySubnet],'''
+                                     primarySubnet,
+                                     securityGroupBindings[
+                                        securityGroup[id, name]]],'''
                 'lastKnownPowerState.name,'
                 'powerState,'
                 'status,'
@@ -296,7 +302,8 @@ class VSManager(utils.IdentifierMixin, object):
             datacenter=None, os_code=None, image_id=None,
             dedicated=False, public_vlan=None, private_vlan=None,
             userdata=None, nic_speed=None, disks=None, post_uri=None,
-            private=False, ssh_keys=None):
+            private=False, ssh_keys=None, public_security_groups=None,
+            private_security_groups=None):
         """Returns a dict appropriate to pass into Virtual_Guest::createObject
 
             See :func:`create_instance` for a list of available options.
@@ -348,6 +355,20 @@ class VSManager(utils.IdentifierMixin, object):
                 "primaryBackendNetworkComponent": {
                     "networkVlan": {"id": int(private_vlan)}}})
 
+        if public_security_groups:
+            secgroups = [{'securityGroup': {'id': int(sg)}}
+                         for sg in public_security_groups]
+            pnc = data.get('primaryNetworkComponent', {})
+            pnc['securityGroupBindings'] = secgroups
+            data.update({'primaryNetworkComponent': pnc})
+
+        if private_security_groups:
+            secgroups = [{'securityGroup': {'id': int(sg)}}
+                         for sg in private_security_groups]
+            pbnc = data.get('primaryBackendNetworkComponent', {})
+            pbnc['securityGroupBindings'] = secgroups
+            data.update({'primaryBackendNetworkComponent': pbnc})
+
         if userdata:
             data['userData'] = [{'value': userdata}]
 
@@ -375,23 +396,20 @@ class VSManager(utils.IdentifierMixin, object):
 
         return data
 
-    def wait_for_transaction(self, instance_id, limit, delay=1):
+    def wait_for_transaction(self, instance_id, limit, delay=10):
         """Waits on a VS transaction for the specified amount of time.
 
         This is really just a wrapper for wait_for_ready(pending=True).
         Provided for backwards compatibility.
 
-
         :param int instance_id: The instance ID with the pending transaction
         :param int limit: The maximum amount of time to wait.
-        :param int delay: The number of seconds to sleep before checks.
-                          Defaults to 1.
+        :param int delay: The number of seconds to sleep before checks. Defaults to 10.
         """
 
-        return self.wait_for_ready(instance_id, limit, delay=delay,
-                                   pending=True)
+        return self.wait_for_ready(instance_id, limit, delay=delay, pending=True)
 
-    def wait_for_ready(self, instance_id, limit, delay=1, pending=False):
+    def wait_for_ready(self, instance_id, limit, delay=10, pending=False):
         """Determine if a VS is ready and available.
 
         In some cases though, that can mean that no transactions are running.
@@ -403,8 +421,7 @@ class VSManager(utils.IdentifierMixin, object):
 
         :param int instance_id: The instance ID with the pending transaction
         :param int limit: The maximum amount of time to wait.
-        :param int delay: The number of seconds to sleep before checks.
-                          Defaults to 1.
+        :param int delay: The number of seconds to sleep before checks. Defaults to 10.
         :param bool pending: Wait for pending transactions not related to
                              provisioning or reloads such as monitoring.
 
@@ -418,36 +435,37 @@ class VSManager(utils.IdentifierMixin, object):
             mask = """id,
                       lastOperatingSystemReload.id,
                       activeTransaction.id,provisionDate"""
-            instance = self.get_instance(new_instance, mask=mask)
-            last_reload = utils.lookup(instance,
-                                       'lastOperatingSystemReload',
-                                       'id')
-            active_transaction = utils.lookup(instance,
-                                              'activeTransaction',
-                                              'id')
+            try:
+                instance = self.get_instance(new_instance, mask=mask)
+                last_reload = utils.lookup(instance, 'lastOperatingSystemReload', 'id')
+                active_transaction = utils.lookup(instance, 'activeTransaction', 'id')
 
-            reloading = all((
-                active_transaction,
-                last_reload,
-                last_reload == active_transaction,
-            ))
+                reloading = all((
+                    active_transaction,
+                    last_reload,
+                    last_reload == active_transaction,
+                ))
 
-            # only check for outstanding transactions if requested
-            outstanding = False
-            if pending:
-                outstanding = active_transaction
+                # only check for outstanding transactions if requested
+                outstanding = False
+                if pending:
+                    outstanding = active_transaction
 
-            # return True if the instance has only if the instance has
-            # finished provisioning and isn't currently reloading the OS.
-            if all([instance.get('provisionDate'),
-                    not reloading,
-                    not outstanding]):
-                return True
+                # return True if the instance has finished provisioning
+                # and isn't currently reloading the OS.
+                if all([instance.get('provisionDate'),
+                        not reloading,
+                        not outstanding]):
+                    return True
+                LOGGER.info("%s not ready.", str(instance_id))
+            except exceptions.SoftLayerAPIError as exception:
+                delay = (delay * 2) + random.randint(0, 9)
+                LOGGER.info('Exception: %s', str(exception))
 
             now = time.time()
             if now >= until:
                 return False
-
+            LOGGER.info('Auto retry in %s seconds', str(min(delay, until - now)))
             time.sleep(min(delay, until - now))
 
     def verify_create_instance(self, **kwargs):
@@ -504,7 +522,8 @@ class VSManager(utils.IdentifierMixin, object):
                 'disks': ('100','25'),
                 'local_disk': True,
                 'memory': 1024,
-                'tags': 'test, pleaseCancel'
+                'tags': 'test, pleaseCancel',
+                'public_security_groups': [12, 15]
             }
 
             vsi = mgr.create_instance(**new_vsi)
@@ -530,6 +549,10 @@ class VSManager(utils.IdentifierMixin, object):
                                incur a fee on your account.
         :param int public_vlan: The ID of the public VLAN on which you want
                                 this VS placed.
+        :param list public_security_groups: The list of security group IDs
+                                            to apply to the public interface
+        :param list private_security_groups: The list of security group IDs
+                                             to apply to the private interface
         :param int private_vlan: The ID of the private VLAN on which you want
                                  this VS placed.
         :param list disks: A list of disk capacities for this server.
@@ -573,7 +596,8 @@ class VSManager(utils.IdentifierMixin, object):
                 'disks': ('100','25'),
                 'local_disk': True,
                 'memory': 1024,
-                'tags': 'test, pleaseCancel'
+                'tags': 'test, pleaseCancel',
+                'public_security_groups': [12, 15]
             }
 
             # using .copy() so we can make changes to individual nodes
